#ifndef WEENSYOS_MEMOS_APP_H
#define WEENSYOS_MEMOS_APP_H
#include "memos.h"


/*****************************************************************************
 * sys_exit(status)
 *
 *   Exit the current process with exit status 'status'.
 *
 *****************************************************************************/

static inline uint32_t
sys_random(void)
{
	// Different system call, different interrupt number (TRAP_SYS_EXIT).
	// This time, we also pass an argument to the system call.
	// We do this by loading the argument into a known register; then
	// the kernel can look up that register value to read the argument.
	// Here, the status is loaded into register %eax.
	// You can load other registers with similar syntax; specifically:
	//	"a" = %eax, "b" = %ebx, "c" = %ecx, "d" = %edx,
	//	"S" = %esi, "D" = %edi.
	uint32_t result;
	asm volatile("int %1\n"
		     : "=a" (result)
		     : "i" (TRAP_SYS_RANDOM)
		     : "cc", "memory");
	return result;
}

static inline int
sys_page_alloc(void *addr)
{
	int r;
	// Different system call, different interrupt number (TRAP_SYS_EXIT).
	// This time, we also pass an argument to the system call.
	// We do this by loading the argument into a known register; then
	// the kernel can look up that register value to read the argument.
	// Here, the status is loaded into register %eax.
	// You can load other registers with similar syntax; specifically:
	//	"a" = %eax, "b" = %ebx, "c" = %ecx, "d" = %edx,
	//	"S" = %esi, "D" = %edi.
	asm volatile("int %1\n"
		     : "=a" (r)
		     : "i" (TRAP_SYS_PAGE_ALLOC),
		       "0" (addr)
		     : "cc", "memory");
	return r;
}

static inline void
sys_yield(void)
{
	// Different system call, different interrupt number (TRAP_SYS_EXIT).
	// This time, we also pass an argument to the system call.
	// We do this by loading the argument into a known register; then
	// the kernel can look up that register value to read the argument.
	// Here, the status is loaded into register %eax.
	// You can load other registers with similar syntax; specifically:
	//	"a" = %eax, "b" = %ebx, "c" = %ecx, "d" = %edx,
	//	"S" = %esi, "D" = %edi.
	asm volatile("int %0\n"
		     : : "i" (TRAP_SYS_YIELD)
		     : "cc", "memory");
}

#endif
