#ifndef WEENSYOS_X86MEM_H
#define WEENSYOS_X86MEM_H

/*****************************************************************************
 * x86mem.h
 *
 *   x86 memory-related constants.
 *
 *****************************************************************************/

#define PAGESIZE	4096		// bytes mapped by a page

// PDX(addr) returns the top 10 bits of the address: the page directory index.
#define PDX(addr)	((((uintptr_t) (addr)) >> 22) & 0x3FF)
// PTX(addr) returns the middle 10 bits of the address: the page table index.
#define PTX(addr)	((((uintptr_t) (addr)) >> 12) & 0x3FF)
// PAGEOFF(addr) returns the bottom 12 bits of the address.
#define PAGEOFF(addr)	(((uintptr_t) (addr)) & 0xFFF)

// PAGENUM(addr) returns the top 20 bits of the address
#define PAGENUM(addr)	(((uintptr_t) (addr)) >> 12)

#define IOPHYSMEM	0x000A0000
#define EXTPHYSMEM	0x00100000

#define CONSOLE_BEGIN	((uint16_t *) 0x000B8000)
#define CONSOLE_END	(CONSOLE_BEGIN + 80 * 25)

// Page table flag bits
#define PTE_P		0x001		// Present
#define PTE_W		0x002		// Writeable
#define PTE_U		0x004		// User
#define PTE_PWT		0x008		// Write-Through
#define PTE_PCD		0x010		// Cache-Disable
#define PTE_A		0x020		// Accessed
#define PTE_D		0x040		// Dirty
#define PTE_PS		0x080		// Page Size
#define PTE_MBZ		0x180		// Bits must be zero
#define PTE_ADDR(pte)	((uint32_t) (pte) & 0xFFFFF000)

#endif /* !WEENSYOS_X86MEM_H */
